//
//  BFRecordScreenCameraManager.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2022/1/13.
//

/*
 
 时长是根据摄像头文件时长相加得来的，预览/合成时需要考虑录音的时长问题
 
 videosticker 已经存储好了，在合成时，需要更新下进入的时间就可以了，不需要再切割了
 
 录音1s失败和摄像头1s失败不同步，需要分开计算
 
 
 
 */

import Foundation
import GPUImage
import BFCommonKit
import BFUIKit
import BFMediaKit
import CoreMedia
import UIKit


let vpath = recordVideosDirectory + "camera_writer.mov"
let videoSize = CGSize(width: cScreenWidth, height: cScreenWidth * 16 / 9.0)

class BFRecordScreenCameraManager : BFRecordScreenBaseManager{
    //
    var rendView = GPUImageView()
    var cameraProgressV: BFCameraProgressView?
    
    // 录制完成回调
    var hasInitCallBack : (()->Void)?
    var recordEndCallBack : recordEndCallBack?
    var recordProgressCallBack : ((CMTime) -> Void)?
    
    var startTime = Date()
    var videoModel = PQEditVisionTrackMaterialsModel()
    
    // 用于打开摄像头初始流程
    static var firstOpenCamera : Bool = true
    
    var currPlayTime : CMTime = .zero
    
    fileprivate var timerr:Timer?
    
    var writerlock = false
    
    // 录制时音频结果
    var recordFinishedResult = false
    var hadDeleteVideoOfFailed = false
    
    var movieWrite : GPUImageMovieWriter?
    
    lazy var camera : GPUImageStillCamera? = {
        let camera = GPUImageStillCamera(sessionPreset: AVCaptureSession.Preset.hd1920x1080.rawValue, cameraPosition: AVCaptureDevice.Position.back)
        camera?.outputImageOrientation = UIInterfaceOrientation.portrait
        camera?.delegate = self
        return camera
        
    }()
    
    var avplayerTimeObserver: NSKeyValueObservation?
    
    lazy var avplayer : AVPlayer = {
        let p = AVPlayer(playerItem: nil)
        avplayerTimeObserver = p.addPeriodicTimeObserver(forInterval: CMTime(value: 1, timescale: 100), queue: DispatchQueue.global(), using: {[weak self] currTime in
            
            guard let wself = self else { return }
            
 
            BFLog(message: "当前播放器状态: \(wself.avplayer.timeControlStatus.rawValue) \(wself.avplayer.currentItem?.status.rawValue)")
            // 拖动的跳跃，不要修改时间点
            if (wself.dele?.isNormalPlaying ?? false){
                wself.currentAssetProgress = CMTime(seconds: (currTime + wself.currPlayTime).seconds, preferredTimescale: 1000)
            }
//            BFLog(1, message: "拖动 currTime: \(currTime.seconds)")
            DispatchQueue.main.async {[weak self] in
                guard let wself = self else { return }
                //add by ak 播放状态才进行刷新 UI
                if(wself.avplayer.timeControlStatus == .playing){
                    wself.dele?.updateSubtitle(time: wself.currentAssetProgress)
                    wself.locationTo(time: wself.currentAssetProgress)
                }

            }
            
        }) as? NSKeyValueObservation
        
        
        return p
    }()
    
    lazy var playerCoverIV : UIImageView = {
        let iv = UIImageView()
        iv.frame = UIScreen.main.bounds
        iv.layer.addSublayer(playerLayer)
        iv.contentMode = .scaleAspectFit
        iv.backgroundColor = .black
        return iv
    }()
    lazy var playerLayer : AVPlayerLayer = {
        let layer = AVPlayerLayer(player: avplayer)
        layer.frame = UIScreen.main.bounds
        return layer
    }()
    
// MARK: 重放
    
    deinit {
        if avplayerTimeObserver != nil {
            avplayer.removeTimeObserver(avplayerTimeObserver as Any)
            avplayerTimeObserver?.invalidate()
        }
    }
    
    override func resetEnv(){
        guard let camera = camera else {
            cShowHUB(superView: nil, msg: "option_fail_camera".BFLocale)
            return
        }
        cameraProgressV?.recordItem = recordItem
        resetThumbImg()
        
        initplayer()

        if let count = recordItem?.voiceStickers.count, count > 0 {
            currentAssetProgress = recordItem!.materialDuraion
            cameraProgressV?.isHidden = false
            progreddL?.isHidden = false
            progreddL?.text = CMTimeGetSeconds(currentAssetProgress).formatDurationToHMS()
            cameraProgressV?.progressView.scrollRectToVisible(CGRect(x: (cameraProgressV?.progressView.contentSize.width)! - 1, y: (cameraProgressV?.progressView.contentSize.height)! - 1, width: 1, height: 1), animated: false)
            dele?.isEndPlay = true
        }else {
            currentAssetProgress = .zero

            cameraProgressV?.isHidden = true
            progreddL?.isHidden = true
            playerCoverIV.isHidden = true
        }
        
        camera.startCapture()
        initerlizeWriter()

        camera.addTarget(filter)
        filter.addTarget(rendView)
        
        if BFRecordScreenCameraManager.firstOpenCamera {
            startRecord()
            DispatchQueue.main.asyncAfter(deadline: .now() + 0.3, execute: { [weak self] in
                guard let wself = self else { return }

                wself.endRecord()
                BFRecordScreenCameraManager.firstOpenCamera = false
            })
        }
        
        super.resetEnv()
    }
    
    func initplayer(){
        if let preView = playView {
            preView.addSubview(playerCoverIV)
        }
    }
    
    override func startRecord(){
        guard let movieWrite = movieWrite, !writerlock else {
            cShowHUB(superView: nil, msg: "option_fail_record".BFLocale)
            return
        }
        
        playerCoverIV.isHidden = true
        recordFinishedResult = false
        
        super.startRecord()

        startTime = Date()
        hadDeleteVideoOfFailed = false
        
        videoModel = PQEditVisionTrackMaterialsModel()
        videoModel.thumImgs = [UIImage]()
        videoModel.timelineCMIn = currentAssetProgress
        videoModel.timelineCMOut = currentAssetProgress
        videoModel.locationPath = "nil"
        
        // 为了第一时间能更新collection view数据
        if !BFRecordScreenCameraManager.firstOpenCamera{
            recordItem?.videoStickers.append(videoModel)
            progreddL?.isHidden = false
            cameraProgressV?.isHidden = false
        }
        movieWrite.startRecording()
        timerr?.invalidate()
        timerr = Timer.scheduledTimer(withTimeInterval: 0.01, repeats: true, block: {[weak self] _ in
            guard let wself = self else { return }
            if BFRecordScreenCameraManager.firstOpenCamera {
                return
            }
            //MARK: 进度计时器

            let currDur = CMTime(seconds: wself.videoModel.timelineCMIn.seconds + movieWrite.duration.seconds, preferredTimescale: 1000)
            if CMTimeCompare(currDur, wself.currentAssetProgress) > 0 {
                wself.currentAssetProgress = currDur
                wself.videoModel.timelineCMOut = currDur
                wself.recordProgressCallBack?(currDur)
                wself.updateUI(progress: currDur)
//                BFLog(1, message: "currTime: \(wself.currentAssetProgress.seconds)")
            }
            
//            wself.getThumImage()
        })
        RunLoop.current.add(timerr!, forMode: .common)
    }
    
    override func endRecord(){
        super.endRecord()

        writerlock = true
        timerr?.invalidate()
        timerr = nil
//        BFLog(1, message: " 拍摄时长 \(Date().timeIntervalSince(startTime))")
        let su = BFRecordScreenCameraManager.firstOpenCamera
        movieWrite?.finishRecording(completionHandler: {[weak self] in
            guard let wself = self else { return }
            
            if FileManager.default.fileExists(atPath: vpath) {
                if !su {
                    let newPath:String? = PQBridgeObject.p_setupFileRename(vpath)
                    if let finalPath = newPath, finalPath.count > 0 {
                        let dur = AVURLAsset(url: URL(fileURLWithPath: finalPath))
                        wself.videoModel.timelineCMOut = wself.videoModel.timelineCMIn + CMTime(seconds: dur.duration.seconds, preferredTimescale: 1000)
                        BFLog(1, message: "文件时长camera:\(wself.recordItem?.videoStickers.count ?? 0),   \(wself.videoModel.timelineCMIn.seconds)~\(wself.videoModel.timelineCMOut.seconds),  \(dur.duration.seconds)")
                        wself.videoModel.locationPath = finalPath
                        if (wself.videoModel.timelineCMOut - wself.videoModel.timelineCMIn).seconds <= 1 {
                            wself.videoModel.locationPath = "nil"
                            wself.revertLast()
                            wself.recordEndCallBack?(false, nil)

                        }else{
                            if wself.recordFinishedResult {
                                // 代表录制成功了
                                wself.currentAssetProgress = wself.videoModel.timelineCMOut
                                wself.recordEndCallBack?(true, wself.videoModel)
                            }
                        }
                    }else{
                        // 录制失败后恢复原样
                        wself.revertLast()
                        wself.recordEndCallBack?(false, nil)
                        cShowHUB(superView: nil, msg: "shoot_tips_least".BFLocale)

                    }
                    DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {[weak self] in
                        guard let wself = self else { return }
                        
                        wself.updateUI(progress:  wself.currentAssetProgress)
                    }

                }
            }
            
            // 重置writer，提前为下次录制做准备
            wself.initerlizeWriter()
            wself.writerlock = false
        })
    }
    
    override func recordFinished(isSuccess:Bool) {

        recordFinishedResult = isSuccess
        
        //
        if videoModel.locationPath != "nil" {
            if isSuccess {
                currentAssetProgress = videoModel.timelineCMOut
                recordEndCallBack?(true, videoModel)
            }else{
                if (videoModel.thumImgs?.count ?? 0) > 0 { //用图片数量来判断是否已经删除了当次录制
                    revertLast()
                    recordEndCallBack?(false, nil)
                }
                BFLog(1, message: "摄像录制小于1s")
            }

        }
    }
    
    func dealCamera(_ isSuccess: Bool) {
        
        
    }
    

    override func deleteRecord(at: CMTime) {
        if let videoS = recordItem?.videoStickers.enumerated().first(where: { mod in
            CMTimeCompare(mod.1.timelineCMIn, at) <= 0 && CMTimeCompare(mod.1.timelineCMOut, at) > 0
        }), let item = recordItem {
            
            item.thumbImgs.removeAll(where: { img in
                videoS.1.thumImgs?.contains(img) ?? false
            })
            item.videoStickers.remove(at: videoS.0)
            
            if var event = dele?.events.last {
                event.deletedCameras = [videoS.1]
                dele?.events.removeLast()
                dele?.events.append(event)
            }
            
            let currDuration = videoS.1.timelineCMOut - videoS.1.timelineCMIn
            currentAssetProgress = videoS.1.timelineCMIn
            // 更新音频的时间点
            for model in item.voiceStickers {
                if CMTimeCompare(currentAssetProgress, model.startCMTime) < 0 {
                    model.startCMTime = model.startCMTime - currDuration
                    model.endCMTime = model.endCMTime - currDuration
                    BFLog(1, message: "camera删除后更新voice time：\(model.startCMTime.seconds) ~ \(model.endCMTime.seconds)")
                    item.titleStickers.forEach { tm in
                        if tm.recordId == model.recordId {
                            tm.timelineIn = tm.timelineIn - currDuration
                            tm.timelineOut = tm.timelineOut - currDuration
                        }
                    }
                }
            }
            // 更新录像的时间点
            for model in item.videoStickers {
                if CMTimeCompare(currentAssetProgress, model.timelineCMIn) < 0 {
                    model.timelineCMIn =  model.timelineCMIn - currDuration
                    model.timelineCMOut = model.timelineCMOut - currDuration
                    BFLog(1, message: "camera删除后更新video time：\(model.timelineCMIn.seconds) ~ \(model.timelineCMOut.seconds)")

                }
            }
            
            item.materialDuraion = item.videoStickers.last?.timelineCMOut ?? .zero

        }
        changeRecordMaterail()
        BFLog(1, message: "delete: video- \(recordItem?.videoStickers.count ?? 0), voice-\(recordItem?.voiceStickers.count ?? 0)")
    }
    
    // 录制失败 ， 不足一秒时恢复上一次状态
    func revertLast(_ modIndex : Int = -1){
        hadDeleteVideoOfFailed = true

        var vmod : PQEditVisionTrackMaterialsModel?
        if modIndex >= 0 && modIndex < recordItem?.videoStickers.count ?? 0{
            vmod = recordItem?.videoStickers[modIndex]
        }else {
//            vmod = recordItem?.videoStickers.first(where: { mod in
//                CMTimeCompare(mod.timelineCMIn, currentAssetProgress) >= 0
//            })
            vmod = videoModel
        }
        if vmod != nil {
            vmod!.timelineCMOut = vmod!.timelineCMIn
            currentAssetProgress = vmod!.timelineCMOut
//            vmod.thumImgs?.removeAll()
            recordItem?.videoStickers.removeAll(where: { mod in
                mod == vmod
            })
            BFLog(1, message: "count videoStickers: \(recordItem?.videoStickers.count ?? 0)")
        }
        
        resetThumbImg()
    }
    
    func resetThumbImg(){
        videoModel.thumImgs?.removeAll()
        let vidos = recordItem?.videoStickers
        var tmpThumbs = [UIImage]()
        vidos?.forEach({ mod in
            if let imgs = mod.thumImgs{
                tmpThumbs.append(contentsOf: imgs)
            }
        })
        recordItem?.thumbImgs = tmpThumbs
        DispatchQueue.main.async {[weak self] in
            guard let wself = self else { return }
            wself.cameraProgressV?.resetThumIV()
        }
        
    }
    
    // 使用avplayer进行播放视频和音频
    override func play() {
        guard playView != nil else {
            return
        }
        
        playerCoverIV.isHidden = false

        camera?.pauseCapture()
        
        // 100ms以内的重头开始播放
        if (recordItem?.materialDuraion.seconds)! - currentAssetProgress.seconds < 0.1 {
            currentAssetProgress = .zero
            currPlayTime = .zero
            locationTo(time: currentAssetProgress)
            avplayer.replaceCurrentItem(with: nil)
        }
        
        prepareToPlayNext()
        
        
    }
    
    func playRecordVoice(needPlay:Bool = true){
        // 播放音频
        if let mod = recordItem?.voiceStickers.first(where: { m in
            CMTimeCompare(m.startCMTime, currentAssetProgress + CMTime(seconds: 0.033, preferredTimescale: 1000)) <= 0 && CMTimeCompare(m.endCMTime, (currentAssetProgress + CMTime(seconds: 0.033, preferredTimescale: 1000))) > 0
        }){
            recordPlayer?.replaceCurrentItem(with: AVPlayerItem(url: URL(fileURLWithPath: mod.wavFilePath)))
            let tt = currentAssetProgress - mod.startCMTime
            BFLog(1, message: "record - tt: \(tt.seconds), curr:\(currentAssetProgress.seconds)")
            if CMTimeCompare(tt, .zero) == 0 {
                if needPlay{
                    recordPlayer?.play()
                }
                return
            }
            recordPlayer?.currentItem?.seek(to: tt, toleranceBefore: CMTime(value: 1, timescale: 1_000_000), toleranceAfter: CMTime(value: 1, timescale: 1_000_000)) { [weak recordPlayer] _ in
                if needPlay{
                    recordPlayer?.play()
                }
            }
        }
    }
    
    func findNextModel() {
        if let mod = recordItem?.videoStickers.sorted(by: { m1, m2 in
            m1.timelineCMIn.seconds < m2.timelineCMIn.seconds
        }).first(where: { mod in
            CMTimeCompare(mod.timelineCMIn, currentAssetProgress + CMTime(seconds: 0.6, preferredTimescale: 1000)) <= 0 && CMTimeCompare((currentAssetProgress + CMTime(seconds: 0.6, preferredTimescale: 1000)), mod.timelineCMOut) < 0
        }){
            currPlayTime = mod.timelineCMIn
            currentAssetProgress = currPlayTime
        }
    }
    
    func prepareToPlayNext(needPlay:Bool = true){
        if let mod = recordItem?.videoStickers.sorted(by: { m1, m2 in
            m1.timelineCMIn.seconds < m2.timelineCMIn.seconds
        }).first(where: { mod in
            CMTimeCompare(mod.timelineCMIn, currentAssetProgress + CMTime(seconds: 0.33, preferredTimescale: 1000)) <= 0 && CMTimeCompare((currentAssetProgress + CMTime(seconds: 0.33, preferredTimescale: 1000)), mod.timelineCMOut) < 0
        }){
            currPlayTime = mod.timelineCMIn
            locationTo(time: currentAssetProgress)
            avplayerReplaceItem(newItem: AVPlayerItem(url: URL(fileURLWithPath: mod.locationPath)))

            let tt = currentAssetProgress - currPlayTime
            BFLog(1, message: "asset - tt: \(tt.seconds), curr:\(currentAssetProgress.seconds)")
            setCoverImage(mod.timelineCMOut)

            avplayer.currentItem?.seek(to: tt, toleranceBefore: CMTime(value: 1, timescale: 1_000_000), toleranceAfter: CMTime(value: 1, timescale: 1_000_000), completionHandler: {[weak avplayer] _ in
                            
                if needPlay {
                    avplayer?.play()
                }
            })
            
            playRecordVoice(needPlay: needPlay)
        }else {
            locationTo(time: currentAssetProgress)
            playerCoverIV.isHidden = true
        }
//        playerCoverIV.isHidden = (CMTimeCompare(currentAssetProgress, recordItem?.materialDuraion ?? .zero) >= 0)
        if !playerCoverIV.isHidden {
            BFLog(1, message: "stor")
        }

    }
    
    func avplayerReplaceItem(newItem: AVPlayerItem?){

        if let item = avplayer.currentItem{
            if (item.asset as? AVURLAsset)?.url == (newItem?.asset as? AVURLAsset)?.url {
                return
            }
            NotificationCenter.default.removeObserver(self as Any, name: .AVPlayerItemDidPlayToEndTime, object: item)
        }
        
        avplayer.replaceCurrentItem(with: newItem)
        if newItem == nil {
            return
        }
        
        NotificationCenter.default.addObserver(forName: .AVPlayerItemDidPlayToEndTime, object: avplayer.currentItem, queue: .main) { [weak self] notice in
            guard let wself = self else {
                return
            }
            
            BFLog(1, message: "endPlay:\(wself.currentAssetProgress.seconds), 总时长:\(wself.recordItem!.materialDuraion.seconds), currplayer:\(wself.avplayer.currentItem?.currentTime().seconds ?? 0)")
            if wself.recordPlayer?.timeControlStatus == .playing{
                wself.recordPlayer?.pause()
            }
            // 如果离结束大于0.7s，代表还有下一段视频
            if ((wself.recordItem?.materialDuraion ?? .zero) - wself.currentAssetProgress).seconds > 0.7 {
                wself.findNextModel()
                wself.prepareToPlayNext()
            }else{
                // 播放到末尾了
                wself.dele?.isEndPlay = true
                wself.dele?.pause()
//                wself.playerLayer.removeFromSuperlayer()
                wself.playerCoverIV.isHidden = true
            }
        }
    }
    
    override func pause() {
        avplayer.pause()
        recordPlayer?.pause()
        if currentAssetProgress == recordItem?.materialDuraion {
            camera?.resumeCameraCapture()
        }
    }
    
    override func dragEnd() {
        // 播放视频
        if let mod = recordItem?.videoStickers.first(where: { m in
            CMTimeCompare(m.timelineCMIn, currentAssetProgress) <= 0 && CMTimeCompare(m.timelineCMOut, currentAssetProgress) > 0
        }){
            currPlayTime = mod.timelineCMIn

            avplayerReplaceItem(newItem: AVPlayerItem(url: URL(fileURLWithPath: mod.locationPath)))
            setCoverImage(mod.timelineCMOut)

            prepareToPlayNext(needPlay: false)
        }
        playerCoverIV.isHidden = (CMTimeCompare(currentAssetProgress, recordItem?.materialDuraion ?? .zero) >= 0)
    }
    
    
    // 获取下一个视频的封面图
    func setCoverImage(_ dur: CMTime){
        if let mod = recordItem?.videoStickers.first(where: { m in
            CMTimeCompare(m.timelineCMIn, (dur + CMTime(seconds: 0.5, preferredTimescale: 1000))) <= 0 && CMTimeCompare(m.timelineCMOut, (dur + CMTime(seconds: 0.5, preferredTimescale: 1000))) > 0
        }){
            playerCoverIV.isHidden = false
            playerCoverIV.image = PQVideoSnapshotUtil.videoSnapshot(videoURL: URL(fileURLWithPath: mod.locationPath), time: .zero)
        }
        playerCoverIV.isHidden = (CMTimeCompare(currentAssetProgress, recordItem?.materialDuraion ?? .zero) >= 0)
    }
    
    override func changeRecordMaterail(){
        //
        if recordItem?.videoStickers.count ?? 0 == 0 {
            currentAssetProgress = .zero
            resetEnv()
        } else {
            cameraProgressV?.resetThumIV()
            cameraProgressV?.progressView.contentOffset = CGPoint(x: currentAssetProgress.seconds * 70.0 / 5.0, y: 0)
            progreddL?.text = CMTimeGetSeconds(currentAssetProgress).formatDurationToHMS()

            cameraProgressV?.isHidden = false
            
            prepareToPlayNext(needPlay: false)
            avplayerReplaceItem(newItem: nil)

        }
    }
    
    override func clean() {
        filter.removeTarget(movieWrite)
        filter.removeTarget(rendView)
        camera?.removeTarget(filter)
        
        camera?.stopCapture()
        
        super.clean()
    }
    
    // MARK: - 私有方法

    // 定位到某个时间点，相应进度条的UI更新
    func locationTo(time:CMTime) {
        var toTime = CMTime(seconds: time.seconds, preferredTimescale: 1000)
        if CMTimeCompare(time, recordItem!.materialDuraion) > 0 {
            toTime = recordItem!.materialDuraion
            currentAssetProgress = toTime
        }
        progreddL?.text = CMTimeGetSeconds(currentAssetProgress).formatDurationToHMS()

        let p = CGPoint(x: currentAssetProgress.seconds * 70.0 / 5.0, y: 0)
        cameraProgressV?.progressView.contentOffset = p
    }
    
    func updateUI(progress:CMTime){
        DispatchQueue.main.async {[weak self] in
            guard let wself = self else { return }
            
            wself.progreddL?.text = CMTimeGetSeconds(progress).formatDurationToHMS()
            if wself.recordItem?.thumbImgs.count ?? 0 > 0 {
                wself.cameraProgressV?.updateCellWidth(index: wself.recordItem!.thumbImgs.count - 1, progress: progress)
            }
        }
    }
    
    // 添加新的缩略图
    func addNewThumb(){
        if let progressThumV = cameraProgressV {
            progressThumV.addThumbImages()
        }
    }
    
    // 初始化 movieWrite
    func initerlizeWriter(){
        if FileManager.default.fileExists(atPath: vpath){
            try? FileManager.default.removeItem(atPath: vpath)
        }
        if movieWrite != nil {
            filter.removeTarget(movieWrite)
            movieWrite = nil
        }
        
        movieWrite = GPUImageMovieWriter(movieURL: URL(fileURLWithPath: vpath), size: videoSize)
        filter.addTarget(movieWrite)

    }
    
}

extension BFRecordScreenCameraManager : GPUImageVideoCameraDelegate {
    func willOutputSampleBuffer(_ sampleBuffer: CMSampleBuffer!) {
        if !recording || BFRecordScreenCameraManager.firstOpenCamera {
            return
        }
        let dur = CMTime(seconds: movieWrite?.duration.seconds ?? 0, preferredTimescale: 1000)
        // 获取缩略图
        let coculationCount = Int(dur.seconds / 5.0)
        if coculationCount >= (videoModel.thumImgs?.count ?? 0) {
            imageFromBuffer(sampleBuffer) { [weak self]  img in
                guard let wself = self else { return }
                
                if img != nil {
//                    BFLog(1, message: "取一张缩略图出来 \(dur.seconds)， \(wself.recordItem?.thumbImgs.count ?? 0)")
                    wself.recordItem?.thumbImgs.append(img!)
                    wself.videoModel.thumImgs?.append(img!)
                    wself.addNewThumb()
                }
            }
        }
    }
    
    func imageFromBuffer(_ buff: CMSampleBuffer, callback: ((UIImage?)->Void)){
        if let buffer = CMSampleBufferGetImageBuffer(buff) {
            let ciImage = CIImage(cvPixelBuffer: buffer)
            let img = UIImage(ciImage: ciImage.oriented(forExifOrientation: 6),
                              scale: cScreenWidth / 70.0,
                              orientation: .up)
            if let data = img.jpegData(compressionQuality: 0.3){
                callback(UIImage(data: data))
            }
        }else{
            callback(nil)
        }
        
    }
    
}
