//
//  BFImageCoverViewCell.swift
//  BFRecordScreenKit
//
//  Created by SanW on 2021/12/15.
//  Copyright © 2021 BytesFlow. All rights reserved.
//

import BFUIKit
import GPUImage
import UIKit

open class BFImageCoverViewCell: UICollectionViewCell {
    var btnClickHandle: ((_ sender: UIButton, _ recordItem: BFRecordItemModel?) -> Void)?

    lazy var playView: GPUImageView = {
        let playView = GPUImageView(frame: bounds)
        return playView
    }()
    
    lazy var playImageView: UIImageView = {
        let playImageView = UIImageView(frame: bounds)
        playImageView.contentMode = .scaleAspectFit
        return playImageView
    }()
    
    lazy var playBtn:UIButton = {
        let btn = UIButton()
        btn.addTarget(self, action: #selector(btnClick(sender:)), for: .touchUpInside)
        return btn
    }()

    lazy var filter: GPUImageFilter = {
        let filter = GPUImageFilter()
        return filter
    }()

    lazy var picture: GPUImagePicture = {
        let picture = GPUImagePicture()
        return picture
    }()

    @objc public class func gpuImageViewCell(collectionView: UICollectionView, indexPath: IndexPath) -> BFImageCoverViewCell {
        let cell: BFImageCoverViewCell = collectionView.dequeueReusableCell(withReuseIdentifier: String(describing: BFImageCoverViewCell.self), for: indexPath) as! BFImageCoverViewCell
        return cell
    }

    override public func prepareForReuse() {
        super.prepareForReuse()
        filter.removeTarget(playView)
        picture.removeTarget(filter)
    }

    override public init(frame: CGRect) {
        super.init(frame: frame)
        contentView.addSubview(playView)
        contentView.addSubview(playBtn)
        contentView.addSubview(playImageView)
    }

    public required init?(coder _: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    public var recordItem: BFRecordItemModel? {
        didSet {
            addData()
            addLayout()
        }
    }

    public func addData() {
        guard let coverImg = recordItem?.coverImg else {
            filter.removeTarget(playView)
            picture.removeTarget(filter)
            return
        }
//        guard let coverImg = UIImage.init(contentsOfFile: recordItem?.coverPath ?? "") ?? recordItem?.coverImg else {
//            filter.removeTarget(playView)
//            picture.removeTarget(filter)
//            return
//        }
//        picture = GPUImagePicture(image: coverImg)
//        picture.addTarget(filter)
//        filter.addTarget(playView)
//        picture.processImage()
        
        playImageView.image = coverImg
        
    }

    public func addLayout() {
        playView.frame = contentView.bounds
        playImageView.frame = contentView.bounds
        playBtn.snp.makeConstraints { make in
            make.left.right.equalToSuperview()
            make.top.equalTo(cDevice_iPhoneNavBarAndStatusBarHei)
            make.bottom.equalTo(-218)
        }
    }

    @objc func btnClick(sender: UIButton) {
        sender.isSelected = !sender.isSelected
        if btnClickHandle != nil {
            btnClickHandle!(sender, recordItem)
        }
    }
    
    func clearImageFilter() {
        filter.removeTarget(playView)
        picture.removeTarget(filter)
        GPUImageContext.sharedFramebufferCache().purgeAllUnassignedFramebuffers()
    }
    
    deinit {
        clearImageFilter()
    }
}
