//
//  BFCameraProgressView.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2022/1/17.
//

import Foundation
import BFCommonKit
import BFUIKit
import CoreMedia
import UIKit

class BFCameraProgressView: BFProgressBaseView {
    
    var imageViews = [BFThumImageView]()
    
    lazy var progressView: BFAutolayoutScrollView = {
        let sv = BFAutolayoutScrollView()
        sv.bounces = false
//        sv.backgroundColor = .clear
//        sv.backgroundColor = UIColor.hexColor(hexadecimal: "#888888",alpha:0.3)
        sv.backgroundColor = UIColor(red: 0, green: 0, blue: 0, alpha: 0.3)
        sv.decelerationRate = .fast
        sv.showsHorizontalScrollIndicator = false
        sv.delegate = self
        return sv
    }()
    
    override init(frame: CGRect) {
        super.init(frame: frame)

//        addSubview(collectionV)
//        insertSubview(collectionV, belowSubview: line)
        
        insertSubview(progressView, belowSubview: line)
    }

    override func layoutSubviews() {
        super.layoutSubviews()

        progressView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }
    }
    
    required init?(coder _: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    /// 添加缩略图
    /// - Parameter images:
    func addThumbImages() {
        DispatchQueue.main.async {[weak self] in
            guard let wself = self else { return }
            
            if let img = wself.recordItem?.thumbImgs.last {
                let iv = BFThumImageView(image: img)
                iv.image = img
                iv.contentMode = .scaleAspectFill
                iv.clipsToBounds = true
                iv.isHiddenBord = true
                wself.progressView.contentView.addSubview(iv)

                if let lastiv = wself.imageViews.last {
                    iv.frame = CGRect(x: lastiv.rightX, y: lastiv.y, width: 0, height: 50)
                    lastiv.isHiddenBord = false
                }else{
                    iv.frame = CGRect(x: cScreenWidth / 2.0, y: 0, width: 0, height: 50)
                }
                wself.imageViews.append(iv)
                BFLog(1, message: "\(wself.imageViews.count), if frame : \(iv.frame)")
                wself.progressView.setNeedsLayout()
                wself.progressView.layoutIfNeeded()
            }
            
        }
        
    }
    
    func resetThumIV() {
        imageViews.forEach { iv in
            iv.removeFromSuperview()
        }
        imageViews.removeAll()
        
        var leftah:CGFloat = cScreenWidth / 2
        if let vstickers = recordItem?.videoStickers {
            for sticker in vstickers {
                if let thumimgs = sticker.thumImgs{
                    for (ind, img) in thumimgs.enumerated(){
                        let iv = BFThumImageView(image: img)
                        iv.image = img
                        iv.contentMode = .scaleAspectFill
                        iv.clipsToBounds = true
                        iv.isHiddenBord = false
                        progressView.contentView.addSubview(iv)
                        imageViews.append(iv)
                        
                        // 计算iv的宽和偏移量
                        let wid = (sticker.timelineCMOut - sticker.timelineCMIn).seconds.truncatingRemainder(dividingBy: 5.0)  * 70.0 / 5.0
                        let wid0 = (ind < (thumimgs.count - 1)) ? 70 : floor(wid * UIScreen.main.scale) /  UIScreen.main.scale
                        let x = CGFloat(ind) * 70.0
                        iv.frame = CGRect(x: leftah + x, y: 0.0, width: wid0, height: 50.0)
                    }
                    leftah = imageViews.last?.rightX ?? 0.0
                }
            }
        }
        
        if let lastiv = imageViews.last {
            lastiv.isHiddenBord = true
        }
        
        let p = CGPoint(x: leftah - cScreenWidth / 2.0, y: 0)

        progressView.contentOffset = p
        progressView.contentSize = CGSize(width: p.x + cScreenWidth, height: 50)
    }
    
    func updateCellWidth(index:Int, progress:CMTime) {
        if let sticker = recordItem?.videoStickers.first(where: { mod in
            CMTimeCompare(mod.timelineCMIn, progress) <= 0 &&  CMTimeCompare(mod.timelineCMOut, progress) >= 0
        }) {
            let wid = (sticker.timelineCMOut - sticker.timelineCMIn).seconds.truncatingRemainder(dividingBy: 5.0)  * 70.0 / 5.0
            let wid0 = floor(wid * UIScreen.main.scale) /  UIScreen.main.scale
            if index < imageViews.count{
                let lastiv = imageViews[index]
                if wid0 <= lastiv.width {
                    return
                }
                var frame = lastiv.frame
                frame.size.width = wid0
                lastiv.frame = frame
            }

            let p = CGPoint(x: progress.seconds * 70 / 5.0, y: 0)
            progressView.contentOffset = p
            progressView.contentSize = CGSize(width: p.x + cScreenWidth, height: 50)
//            BFLog(1, message: "contentSize: \(progressView.contentSize)")
        }
    }
}

extension BFCameraProgressView: UIScrollViewDelegate {
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        if isDrag {
            dragScrollProgressHandle?(false, scorllToProgress(scrollView))
        }
    }

    func scrollViewWillBeginDragging(_ scrollView: UIScrollView) {
        isDrag = true
        dragStartHandle?()
        dragScrollProgressHandle?(true, scorllToProgress(scrollView))
    }

    func scrollViewDidEndDragging(_ scrollView: UIScrollView, willDecelerate decelerate: Bool) {
        if !decelerate {
            isDrag = false
            dragEndHandle?(scorllToProgress(scrollView))
        }
    }

    func scrollViewDidEndDecelerating(_ scrollView: UIScrollView) {
        isDrag = false
        dragEndHandle?(scorllToProgress(scrollView))
    }
    
    func scorllToProgress(_ scrollView: UIScrollView) -> Float {
        let totalW = Float(Int(scrollView.contentOffset.x * UIScreen.main.scale)) / Float(Int((scrollView.contentSize.width - cScreenWidth) * UIScreen.main.scale))
        return Float(min(1, max(0, totalW)))
    }
}
